@@ -47,6 +47,8 @@ def flake8ext(f):
     f.version = '0.0.1'
     return f
 
+# Error code block layout
+
 #H1xx comments
 #H2xx except
 #H3xx imports
@@ -87,14 +89,21 @@ def import_normalize(line):
 def hacking_todo_format(physical_line, tokens):
     """Check for 'TODO()'.
 
-    nova HACKING guide recommendation for TODO:
+    OpenStack HACKING guide recommendation for TODO:
     Include your name with TODOs as in "#TODO(termie)"
 
     Okay: #TODO(sdague)
+    Okay: # TODO(sdague)
     H101: #TODO fail
+    H101: #TODO
     H101: #TODO (jogo) fail
+    Okay: TODO = 5
     """
-    # TODO(sdague): TODO check shouldn't fail inside of space
+    #TODO(jogo): make the following doctests pass:
+    #            H101: #TODO(jogo fail
+    #            H101: #TODO(jogo
+    #TODO(jogo): make this check docstrings as well (don't have to be at top
+    # of function)
     pos = physical_line.find('TODO')
     pos1 = physical_line.find('TODO(')
     pos2 = physical_line.find('#')  # make sure it's a comment
@@ -102,14 +111,97 @@ def hacking_todo_format(physical_line, tokens):
         return pos, "H101: Use TODO(NAME)"
 
 
+def _check_for_apache(start, lines):
+    """Check for the Apache 2.0 license header.
+
+    We strip all the newlines and extra spaces so this license string
+    should work regardless of indentation in the file.
+    """
+    APACHE2 = """
+Licensed under the Apache License, Version 2.0 (the "License"); you may
+not use this file except in compliance with the License. You may obtain
+a copy of the License at
+
+http://www.apache.org/licenses/LICENSE-2.0
+
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
+WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
+License for the specific language governing permissions and limitations
+under the License."""
+
+    # out of all the formatting I've seen, a 12 line version seems to be the
+    # longest in the source tree. So just take the 12 lines starting with where
+    # the Apache starting words were found, strip all the '#' and collapse the
+    # spaces.
+    content = ''.join(lines[start:(start + 12)])
+    content = re.sub('\#', '', content)
+    content = re.sub('\s+', ' ', content)
+    stripped_apache2 = re.sub('\s+', ' ', APACHE2)
+
+    if stripped_apache2 in content:
+        return True
+    else:
+        print "License '%s' != '%s'" % (stripped_apache2, content)
+        return False
+
+
+def _project_is_apache():
+    """Determine if a project is Apache.
+
+    Look for a key string in a set of possible license files to figure out
+    if a project looks to be Apache. This is used as a precondition for
+    enforcing license headers.
+    """
+
+    license_files = ["LICENSE"]
+    for filename in license_files:
+        try:
+            with open(filename, "r") as file:
+                for line in file:
+                    if re.search('Apache License', line):
+                        return True
+        except IOError:
+            pass
+    return False
+
+
+@flake8ext
+def hacking_has_license(physical_line, filename, lines, line_number):
+    """Check for Apache 2.0 license.
+
+    H102
+    """
+    # don't work about init files for now
+    # TODO(sdague): enforce license in init file if it's not empty of content
+    license_found = False
+
+    # skip files that are < 10 lines, which isn't enough for a license to fit
+    # this allows us to handle empty files, as well as not fail on the Okay
+    # doctests.
+    if _project_is_apache() and not line_number > 1 and len(lines) > 10:
+        for idx, line in enumerate(lines):
+            # if it's more than 10 characters in, it's probably not in the
+            # header
+            if 0 < line.find('Licensed under the Apache License') < 10:
+                if _check_for_apache(idx, lines):
+                    license_found = True
+                else:
+                    return (idx, "H102: Apache 2.0 license header not found")
+
+        if not license_found:
+            return (0, "H102: Apache 2.0 license header not found")
+
+
 @flake8ext
 def hacking_except_format(logical_line):
     r"""Check for 'except:'.
 
-    nova HACKING guide recommends not using except:
+    OpenStack HACKING guide recommends not using except:
     Do not write "except:", use "except Exception:" at the very least
 
     Okay: try:\n    pass\nexcept Exception:\n    pass
+    H201: try:\n    pass\nexcept:\n    pass
     H201: except:
     """
     if logical_line.startswith("except:"):
@@ -120,13 +212,14 @@ def hacking_except_format(logical_line):
 def hacking_except_format_assert(logical_line):
     r"""Check for 'assertRaises(Exception'.
 
-    nova HACKING guide recommends not using assertRaises(Exception...):
+    OpenStack HACKING guide recommends not using assertRaises(Exception...):
     Do not use overly broad Exception type
 
     Okay: self.assertRaises(NovaException)
+    Okay: self.assertRaises(ExceptionStrangeNotation)
     H202: self.assertRaises(Exception)
     """
-    if logical_line.startswith("self.assertRaises(Exception"):
+    if logical_line.startswith("self.assertRaises(Exception)"):
         yield 1, "H202: assertRaises Exception too broad"
 
 
@@ -140,7 +233,7 @@ RE_RELATIVE_IMPORT = re.compile('^from\s*[.]')
 def hacking_import_rules(logical_line, filename):
     r"""Check for imports.
 
-    nova HACKING guide recommends one import per line:
+    OpenStack HACKING guide recommends one import per line:
     Do not import more than one module per line
 
     Examples:
@@ -150,7 +243,7 @@ def hacking_import_rules(logical_line, filename):
 
     Imports should usually be on separate lines.
 
-    nova HACKING guide recommends importing only modules:
+    OpenStack HACKING guide recommends importing only modules:
     Do not import objects, only modules
 
     Examples:
@@ -164,6 +257,8 @@ def hacking_import_rules(logical_line, filename):
     H303: from os.path import *
     H304: from .compute import rpcapi
     """
+    #TODO(jogo): make the following doctests pass:
+    #            H301: import os, sys
     #NOTE(afazekas): An old style relative import example will not be able to
     # pass the doctest, since the relativity depends on the file's locality
     #TODO(mordred: We need to split this into 4 different checks so that they
@@ -262,6 +357,7 @@ def hacking_import_alphabetical(logical_line, blank_lines, previous_logical,
     imports in human alphabetical order
 
     Okay: import os\nimport sys\n\nimport nova\nfrom nova import test
+    Okay: import os\nimport sys
     H306: import sys\nimport os
     """
     # handle import x
@@ -280,6 +376,7 @@ def hacking_import_alphabetical(logical_line, blank_lines, previous_logical,
 
 def is_docstring(physical_line, previous_logical):
     """Return True if found docstring
+
     'A docstring is a string literal that occurs as the first statement in a
     module, function, class,'
     http://www.python.org/dev/peps/pep-0257/#what-is-a-docstring
@@ -298,9 +395,9 @@ def is_docstring(physical_line, previous_logical):
 
 @flake8ext
 def hacking_docstring_start_space(physical_line, previous_logical):
-    r"""Check for docstring not start with space.
+    r"""Check for docstring not starting with space.
 
-    nova HACKING guide recommendation for docstring:
+    OpenStack HACKING guide recommendation for docstring:
     Docstring should not start with space
 
     Okay: def foo():\n    '''This is good.'''
@@ -327,7 +424,7 @@ def hacking_docstring_start_space(physical_line, previous_logical):
 def hacking_docstring_one_line(physical_line, previous_logical):
     r"""Check one line docstring end.
 
-    nova HACKING guide recommendation for one line docstring:
+    OpenStack HACKING guide recommendation for one line docstring:
     A one line docstring looks like this and ends in punctuation.
 
     Okay: def foo():\n    '''This is good.'''
@@ -338,6 +435,8 @@ def hacking_docstring_one_line(physical_line, previous_logical):
     Okay: class Foo:\n    pass\n    '''This is not a docstring'''
     H402: def foo():\n    '''This is not'''
     H402: def foo():\n    '''Bad punctuation,'''
+    H402: def foo():\n    '''Bad punctuation:'''
+    H402: def foo():\n    '''Bad punctuation;'''
     H402: class Foo:\n    '''Bad punctuation,'''
     """
     #TODO(jogo) make this apply to multi line docstrings as well
@@ -355,7 +454,7 @@ def hacking_docstring_one_line(physical_line, previous_logical):
 def hacking_docstring_multiline_end(physical_line, previous_logical, tokens):
     r"""Check multi line docstring end.
 
-    nova HACKING guide recommendation for docstring:
+    OpenStack HACKING guide recommendation for docstring:
     Docstring should end on a new line
 
     Okay: '''foobar\nfoo\nbar\n'''
@@ -379,7 +478,7 @@ def hacking_docstring_multiline_end(physical_line, previous_logical, tokens):
 def hacking_docstring_multiline_start(physical_line, previous_logical, tokens):
     r"""Check multi line docstring start with summary.
 
-    nova HACKING guide recommendation for docstring:
+    OpenStack HACKING guide recommendation for docstring:
     Docstring should start with A multi line docstring has a one-line summary
 
     Okay: '''foobar\nfoo\nbar\n'''
@@ -404,6 +503,7 @@ def hacking_no_cr(physical_line):
     # upstream fix
     H601 import os\r\nimport sys
     """
+    #TODO(jogo): This should not be in H6xx, it should be in H9xx
     pos = physical_line.find('\r')
     if pos != -1 and pos == (len(physical_line) - 2):
         return (pos, "H601: Windows style line endings not allowed in code")
@@ -490,6 +590,7 @@ def hacking_localization_strings(logical_line, tokens):
 
     Okay: _("This is fine")
     Okay: _("This is also fine %s")
+    Okay: _("So is this %s, %(foo)s") % {foo: 'foo'}
     H701: _('')
     H702: _("Bob" + " foo")
     H702: _("Bob %s" % foo)
@@ -510,7 +611,7 @@ def hacking_localization_strings(logical_line, tokens):
 
 @flake8ext
 def hacking_is_not(logical_line):
-    r"""Check localization in line.
+    r"""Check for use of 'is not' for testing unequal identities.
 
     Okay: if x is not y:\n    pass
     H901: if not X is Y
@@ -520,12 +621,12 @@ def hacking_is_not(logical_line):
     if (len(split_line) == 5 and split_line[0] == 'if' and
             split_line[1] == 'not' and split_line[3] == 'is'):
                 yield (logical_line.find('not'), "H901: Use the 'is not' "
-                       "operator for when testing for unequal identities")
+                       "operator when testing for unequal identities")
 
 
 @flake8ext
 def hacking_not_in(logical_line):
-    r"""Check localization in line.
+    r"""Check for use of "not in" for evaluating membership.
 
     Okay: if x not in y:\n    pass
     Okay: if not (X in Y or X is Z):\n    pass
@@ -540,13 +641,16 @@ def hacking_not_in(logical_line):
                 yield (logical_line.find('not'), "H902: Use the 'not in' "
                        "operator for collection membership evaluation")
 
-_has_run_registry = dict()
-
 
 class GlobalCheck(object):
     """Base class for checks that should be run only once."""
 
+    name = None
     version = '0.0.1'
+    _has_run = set()
+
+    def __init__(self, tree, *args):
+        pass
 
     def run(self):
         """Make run a no-op if run() has been called before.
@@ -556,30 +660,18 @@ class GlobalCheck(object):
         This way, since pep8 is file/line based, we don't wind up re-running
         a check on a git commit message over and over again.
         """
-        global _has_run_registry
-        if self.name not in _has_run_registry.keys():
-            _has_run_registry[self.name] = True
+        if self.name and self.name not in self.__class__._has_run:
+            self.__class__._has_run.add(self.name)
             ret = self.run_once()
             if ret is not None:
-                for r in ret:
-                    yield ret
+                yield ret
 
     def run_once(self):
         pass
 
 
 class GitCheck(GlobalCheck):
-
-    #From https://github.com/openstack/openstack-ci-puppet
-    #       /blob/master/modules/gerrit/manifests/init.pp#L74
-    #Changeid|bug|blueprint
-    git_keywords = (r'(I[0-9a-f]{8,40})|'
-                    '([Bb]ug|[Ll][Pp])[\s\#:]*(\d+)|'
-                    '([Bb]lue[Pp]rint|[Bb][Pp])[\s\#:]*([A-Za-z0-9\\-]+)')
-    GIT_REGEX = re.compile(git_keywords)
-
-    def __init__(self, tree, *args):
-        pass
+    """Base-class for Git related checks."""
 
     def _get_commit_title(self):
         if not os.path.exists('.git'):
@@ -598,14 +690,19 @@ class GitCheck(GlobalCheck):
 class OnceGitCheckCommitTitleBug(GitCheck):
     """Check git commit messages for bugs.
 
-    nova HACKING recommends not referencing a bug or blueprint in first line,
-    it should provide an accurate description of the change
+    OpenStack HACKING recommends not referencing a bug or blueprint in first
+    line. It should provide an accurate description of the change
     H801
     """
     name = "GitCheckCommitTitleBug"
 
-    def __init__(self, tree, *args):
-        pass
+    #From https://github.com/openstack/openstack-ci-puppet
+    #       /blob/master/modules/gerrit/manifests/init.pp#L74
+    #Changeid|bug|blueprint
+    GIT_REGEX = re.compile(
+        r'(I[0-9a-f]{8,40})|'
+        '([Bb]ug|[Ll][Pp])[\s\#:]*(\d+)|'
+        '([Bb]lue[Pp]rint|[Bb][Pp])[\s\#:]*([A-Za-z0-9\\-]+)')
 
     def run_once(self):
         title = self._get_commit_title()
@@ -613,10 +710,10 @@ class OnceGitCheckCommitTitleBug(GitCheck):
         #NOTE(jogo) if match regex but over 3 words, acceptable title
         if (title and self.GIT_REGEX.search(title) is not None
                 and len(title.split()) <= 3):
-            return(1, 0,
-                   "H801: git commit title ('%s') should provide an accurate "
-                   "description of the change, not just a reference to a bug "
-                   "or blueprint" % title.strip(), self.name)
+            return (1, 0,
+                    "H801: git commit title ('%s') should provide an accurate "
+                    "description of the change, not just a reference to a bug "
+                    "or blueprint" % title.strip(), self.name)
 
 
 class OnceGitCheckCommitTitleLength(GitCheck):
@@ -629,14 +726,11 @@ class OnceGitCheckCommitTitleLength(GitCheck):
     """
     name = "GitCheckCommitTitleLength"
 
-    def __init__(self, tree, *args):
-        pass
-
     def run_once(self):
         title = self._get_commit_title()
 
         if title and len(title.decode('utf-8')) > 72:
-            return(
+            return (
                 1, 0,
                 "H802: git commit title ('%s') should be under 50 chars"
                 % title.strip(),
@@ -645,12 +739,8 @@ class OnceGitCheckCommitTitleLength(GitCheck):
 
 class ProxyChecks(GlobalCheck):
     """Provide a mechanism for locally defined checks."""
-
     name = 'ProxyChecker'
 
-    def __init__(self, tree, *args):
-        pass
-
     @classmethod
     def add_options(cls, parser):
         # Abusing this method because of when it gets called
